@@ -1,22 +1,10 @@
-/***************************************************************************
- *   Copyright 2010  Thomas Baumgart  tbaumgart@kde.org                    *
- *   Copyright 2018  Thomas Baumgart  tbaumgart@kde.org                    *
- *                                                                         *
- *   This file is part of libalkimia.                                      *
- *                                                                         *
- *   libalkimia is free software; you can redistribute it and/or           *
- *   modify it under the terms of the GNU Lesser General Public License    *
- *   as published by the Free Software Foundation; either version 2.1 of   *
- *   the License or (at your option) version 3 or any later version.       *
- *                                                                         *
- *   libalkimia is distributed in the hope that it will be useful,         *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2010-2021 Thomas Baumgart tbaumgart @kde.org
+
+    This file is part of libalkimia.
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #include "alkimia/alkvalue.h"
 
@@ -256,7 +244,13 @@ double AlkValue::toDouble() const
     return d->m_val.get_d();
 }
 
-AlkValue AlkValue::convertDenominator(int _denom, const RoundingMethod how) const
+AlkValue AlkValue::convertDenominator(int _denom, const AlkValue::RoundingMethod how) const
+{
+    mpz_class denom(_denom);
+    return convertDenominator(denom, how);
+}
+
+AlkValue AlkValue::convertDenominator(const mpz_class _denom, const AlkValue::RoundingMethod how) const
 {
     AlkValue in(*this);
     mpz_class in_num(mpq_numref(in.d->m_val.get_mpq_t()));
@@ -268,7 +262,7 @@ AlkValue AlkValue::convertDenominator(int _denom, const RoundingMethod how) cons
         // sign is either -1 for negative numbers or +1 in all other cases
 
         AlkValue temp;
-        mpz_class denom = _denom;
+        mpz_class denom(_denom);
         // only process in case the denominators are different
         if (mpz_cmpabs(denom.get_mpz_t(), mpq_denref(d->m_val.get_mpq_t())) != 0) {
             mpz_class in_denom(mpq_denref(in.d->m_val.get_mpq_t()));
@@ -367,8 +361,8 @@ AlkValue AlkValue::convertDenominator(int _denom, const RoundingMethod how) cons
                     break;
 
                 case RoundNever:
-                    qWarning("AlkValue: have remainder \"%s\"->convert(%d, %d)",
-                             qPrintable(toString()), _denom, how);
+                    qWarning("AlkValue: have remainder \"%s\"->convert(%s, %d)",
+                             qPrintable(toString()), denom.get_str().c_str(), how);
                     break;
                 }
             }
